@@ -1,4 +1,4 @@
 # vim: set filetype=python ts=4 sw=4
 # -*- coding: utf-8 -*-
 """tokendito module initialization."""
-__all__ = [""]
+__all__ = []
